import datetime
from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.status_mounted_status import StatusMountedStatus

T = TypeVar("T", bound="StatusMounted")


@_attrs_define
class StatusMounted:
    """
    Attributes:
        status (StatusMountedStatus):
        initialized (datetime.datetime):
        write_start (datetime.datetime):
        mounted (datetime.datetime):
    """

    status: StatusMountedStatus
    initialized: datetime.datetime
    write_start: datetime.datetime
    mounted: datetime.datetime
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        status = self.status.value

        initialized = self.initialized.isoformat()

        write_start = self.write_start.isoformat()

        mounted = self.mounted.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "status": status,
                "initialized": initialized,
                "writeStart": write_start,
                "mounted": mounted,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        status = StatusMountedStatus(d.pop("status"))

        initialized = isoparse(d.pop("initialized"))

        write_start = isoparse(d.pop("writeStart"))

        mounted = isoparse(d.pop("mounted"))

        status_mounted = cls(
            status=status,
            initialized=initialized,
            write_start=write_start,
            mounted=mounted,
        )

        status_mounted.additional_properties = d
        return status_mounted

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .algo_log import AlgoLog
from .algo_log_log import AlgoLogLog
from .collaborator_status import CollaboratorStatus
from .finished_report import FinishedReport
from .finished_report_error_item import FinishedReportErrorItem
from .finished_report_fail_item import FinishedReportFailItem
from .finished_report_status import FinishedReportStatus
from .finished_report_success_item import FinishedReportSuccessItem
from .finished_summary import FinishedSummary
from .finished_summary_status import FinishedSummaryStatus
from .get_attestation_token_nonce import GetAttestationTokenNonce
from .get_collaborator_status_response_404 import GetCollaboratorStatusResponse404
from .get_report_response_404 import GetReportResponse404
from .mount_collaborator_response_400 import MountCollaboratorResponse400
from .mount_collaborator_response_404 import MountCollaboratorResponse404
from .pending_report import PendingReport
from .pending_report_status import PendingReportStatus
from .post_event_body import PostEventBody
from .post_event_response_200 import PostEventResponse200
from .post_event_response_400 import PostEventResponse400
from .put_secrets_secret_body import PutSecretsSecretBody
from .put_secrets_secret_response_400 import PutSecretsSecretResponse400
from .start_quality_validation_response_201 import StartQualityValidationResponse201
from .start_quality_validation_response_400 import StartQualityValidationResponse400
from .start_quality_validation_response_404 import StartQualityValidationResponse404
from .status_error import StatusError
from .status_exported import StatusExported
from .status_exported_status import StatusExportedStatus
from .status_exporting import StatusExporting
from .status_exporting_status import StatusExportingStatus
from .status_initialized import StatusInitialized
from .status_initialized_status import StatusInitializedStatus
from .status_mounted import StatusMounted
from .status_mounted_status import StatusMountedStatus
from .status_unmounted import StatusUnmounted
from .status_writing import StatusWriting
from .status_writing_status import StatusWritingStatus
from .unmount_collaborator_response_404 import UnmountCollaboratorResponse404

__all__ = (
    "AlgoLog",
    "AlgoLogLog",
    "CollaboratorStatus",
    "FinishedReport",
    "FinishedReportErrorItem",
    "FinishedReportFailItem",
    "FinishedReportStatus",
    "FinishedReportSuccessItem",
    "FinishedSummary",
    "FinishedSummaryStatus",
    "GetAttestationTokenNonce",
    "GetCollaboratorStatusResponse404",
    "GetReportResponse404",
    "MountCollaboratorResponse400",
    "MountCollaboratorResponse404",
    "PendingReport",
    "PendingReportStatus",
    "PostEventBody",
    "PostEventResponse200",
    "PostEventResponse400",
    "PutSecretsSecretBody",
    "PutSecretsSecretResponse400",
    "StartQualityValidationResponse201",
    "StartQualityValidationResponse400",
    "StartQualityValidationResponse404",
    "StatusError",
    "StatusExported",
    "StatusExportedStatus",
    "StatusExporting",
    "StatusExportingStatus",
    "StatusInitialized",
    "StatusInitializedStatus",
    "StatusMounted",
    "StatusMountedStatus",
    "StatusUnmounted",
    "StatusWriting",
    "StatusWritingStatus",
    "UnmountCollaboratorResponse404",
)



import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.finished_report_status import FinishedReportStatus

if TYPE_CHECKING:
    from ..models.finished_report_error_item import FinishedReportErrorItem
    from ..models.finished_report_fail_item import FinishedReportFailItem
    from ..models.finished_report_success_item import FinishedReportSuccessItem


T = TypeVar("T", bound="FinishedReport")


@_attrs_define
class FinishedReport:
    """
    Attributes:
        id (str):
        status (FinishedReportStatus):
        collaborator_id (str):
        mount (datetime.datetime):
        start (datetime.datetime):
        finish (datetime.datetime):
        success (list['FinishedReportSuccessItem']):
        fail (list['FinishedReportFailItem']):
        error (list['FinishedReportErrorItem']):
    """

    id: str
    status: FinishedReportStatus
    collaborator_id: str
    mount: datetime.datetime
    start: datetime.datetime
    finish: datetime.datetime
    success: list["FinishedReportSuccessItem"]
    fail: list["FinishedReportFailItem"]
    error: list["FinishedReportErrorItem"]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        status = self.status.value

        collaborator_id = self.collaborator_id

        mount = self.mount.isoformat()

        start = self.start.isoformat()

        finish = self.finish.isoformat()

        success = []
        for success_item_data in self.success:
            success_item = success_item_data.to_dict()
            success.append(success_item)

        fail = []
        for fail_item_data in self.fail:
            fail_item = fail_item_data.to_dict()
            fail.append(fail_item)

        error = []
        for error_item_data in self.error:
            error_item = error_item_data.to_dict()
            error.append(error_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "status": status,
                "collaboratorId": collaborator_id,
                "mount": mount,
                "start": start,
                "finish": finish,
                "success": success,
                "fail": fail,
                "error": error,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.finished_report_error_item import FinishedReportErrorItem
        from ..models.finished_report_fail_item import FinishedReportFailItem
        from ..models.finished_report_success_item import FinishedReportSuccessItem

        d = dict(src_dict)
        id = d.pop("id")

        status = FinishedReportStatus(d.pop("status"))

        collaborator_id = d.pop("collaboratorId")

        mount = isoparse(d.pop("mount"))

        start = isoparse(d.pop("start"))

        finish = isoparse(d.pop("finish"))

        success = []
        _success = d.pop("success")
        for success_item_data in _success:
            success_item = FinishedReportSuccessItem.from_dict(success_item_data)

            success.append(success_item)

        fail = []
        _fail = d.pop("fail")
        for fail_item_data in _fail:
            fail_item = FinishedReportFailItem.from_dict(fail_item_data)

            fail.append(fail_item)

        error = []
        _error = d.pop("error")
        for error_item_data in _error:
            error_item = FinishedReportErrorItem.from_dict(error_item_data)

            error.append(error_item)

        finished_report = cls(
            id=id,
            status=status,
            collaborator_id=collaborator_id,
            mount=mount,
            start=start,
            finish=finish,
            success=success,
            fail=fail,
            error=error,
        )

        finished_report.additional_properties = d
        return finished_report

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="AlgoLogLog")


@_attrs_define
class AlgoLogLog:
    """
    Attributes:
        msg (str):
        library_version (str):
        event (Union[Unset, str]):
        event_stream (Union[Unset, str]):
    """

    msg: str
    library_version: str
    event: Union[Unset, str] = UNSET
    event_stream: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        msg = self.msg

        library_version = self.library_version

        event = self.event

        event_stream = self.event_stream

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "msg": msg,
                "libraryVersion": library_version,
            }
        )
        if event is not UNSET:
            field_dict["event"] = event
        if event_stream is not UNSET:
            field_dict["eventStream"] = event_stream

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        msg = d.pop("msg")

        library_version = d.pop("libraryVersion")

        event = d.pop("event", UNSET)

        event_stream = d.pop("eventStream", UNSET)

        algo_log_log = cls(
            msg=msg,
            library_version=library_version,
            event=event,
            event_stream=event_stream,
        )

        algo_log_log.additional_properties = d
        return algo_log_log

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.finished_summary import FinishedSummary
    from ..models.pending_report import PendingReport
    from ..models.status_error import StatusError
    from ..models.status_exported import StatusExported
    from ..models.status_exporting import StatusExporting
    from ..models.status_initialized import StatusInitialized
    from ..models.status_mounted import StatusMounted
    from ..models.status_unmounted import StatusUnmounted
    from ..models.status_writing import StatusWriting


T = TypeVar("T", bound="CollaboratorStatus")


@_attrs_define
class CollaboratorStatus:
    """
    Attributes:
        status (Union['StatusError', 'StatusExported', 'StatusExporting', 'StatusInitialized', 'StatusMounted',
            'StatusUnmounted', 'StatusWriting']):
        reports (list[Union['FinishedSummary', 'PendingReport']]): The summarized quality reports of the collaborator.
        has_server_secret (bool): Indicates if a secret has been loaded to push/pull the data.
    """

    status: Union[
        "StatusError",
        "StatusExported",
        "StatusExporting",
        "StatusInitialized",
        "StatusMounted",
        "StatusUnmounted",
        "StatusWriting",
    ]
    reports: list[Union["FinishedSummary", "PendingReport"]]
    has_server_secret: bool
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.pending_report import PendingReport
        from ..models.status_exported import StatusExported
        from ..models.status_exporting import StatusExporting
        from ..models.status_initialized import StatusInitialized
        from ..models.status_mounted import StatusMounted
        from ..models.status_unmounted import StatusUnmounted
        from ..models.status_writing import StatusWriting

        status: dict[str, Any]
        if isinstance(self.status, StatusUnmounted):
            status = self.status.to_dict()
        elif isinstance(self.status, StatusInitialized):
            status = self.status.to_dict()
        elif isinstance(self.status, StatusWriting):
            status = self.status.to_dict()
        elif isinstance(self.status, StatusMounted):
            status = self.status.to_dict()
        elif isinstance(self.status, StatusExporting):
            status = self.status.to_dict()
        elif isinstance(self.status, StatusExported):
            status = self.status.to_dict()
        else:
            status = self.status.to_dict()

        reports = []
        for reports_item_data in self.reports:
            reports_item: dict[str, Any]
            if isinstance(reports_item_data, PendingReport):
                reports_item = reports_item_data.to_dict()
            else:
                reports_item = reports_item_data.to_dict()

            reports.append(reports_item)

        has_server_secret = self.has_server_secret

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "status": status,
                "reports": reports,
                "hasServerSecret": has_server_secret,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.finished_summary import FinishedSummary
        from ..models.pending_report import PendingReport
        from ..models.status_error import StatusError
        from ..models.status_exported import StatusExported
        from ..models.status_exporting import StatusExporting
        from ..models.status_initialized import StatusInitialized
        from ..models.status_mounted import StatusMounted
        from ..models.status_unmounted import StatusUnmounted
        from ..models.status_writing import StatusWriting

        d = dict(src_dict)

        def _parse_status(
            data: object,
        ) -> Union[
            "StatusError",
            "StatusExported",
            "StatusExporting",
            "StatusInitialized",
            "StatusMounted",
            "StatusUnmounted",
            "StatusWriting",
        ]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_status_type_0 = StatusUnmounted.from_dict(data)

                return componentsschemas_status_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_status_type_1 = StatusInitialized.from_dict(data)

                return componentsschemas_status_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_status_type_2 = StatusWriting.from_dict(data)

                return componentsschemas_status_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_status_type_3 = StatusMounted.from_dict(data)

                return componentsschemas_status_type_3
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_status_type_4 = StatusExporting.from_dict(data)

                return componentsschemas_status_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_status_type_5 = StatusExported.from_dict(data)

                return componentsschemas_status_type_5
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemas_status_type_6 = StatusError.from_dict(data)

            return componentsschemas_status_type_6

        status = _parse_status(d.pop("status"))

        reports = []
        _reports = d.pop("reports")
        for reports_item_data in _reports:

            def _parse_reports_item(data: object) -> Union["FinishedSummary", "PendingReport"]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_quality_report_summary_type_0 = PendingReport.from_dict(data)

                    return componentsschemas_quality_report_summary_type_0
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_quality_report_summary_type_1 = FinishedSummary.from_dict(data)

                return componentsschemas_quality_report_summary_type_1

            reports_item = _parse_reports_item(reports_item_data)

            reports.append(reports_item)

        has_server_secret = d.pop("hasServerSecret")

        collaborator_status = cls(
            status=status,
            reports=reports,
            has_server_secret=has_server_secret,
        )

        collaborator_status.additional_properties = d
        return collaborator_status

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.finished_report import FinishedReport
from ...models.get_report_response_404 import GetReportResponse404
from ...models.pending_report import PendingReport
from ...types import Response


def _get_kwargs(
    report_id: str,
) -> dict[str, Any]:
    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": f"/quality-reports/{report_id}",
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[GetReportResponse404, Union["FinishedReport", "PendingReport"]]]:
    if response.status_code == 200:

        def _parse_response_200(data: object) -> Union["FinishedReport", "PendingReport"]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_quality_report_type_0 = PendingReport.from_dict(data)

                return componentsschemas_quality_report_type_0
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemas_quality_report_type_1 = FinishedReport.from_dict(data)

            return componentsschemas_quality_report_type_1

        response_200 = _parse_response_200(response.json())

        return response_200
    if response.status_code == 404:
        response_404 = GetReportResponse404.from_dict(response.json())

        return response_404
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[GetReportResponse404, Union["FinishedReport", "PendingReport"]]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    report_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[Union[GetReportResponse404, Union["FinishedReport", "PendingReport"]]]:
    """Get Report

     Get a quality report by ID

    Args:
        report_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[GetReportResponse404, Union['FinishedReport', 'PendingReport']]]
    """

    kwargs = _get_kwargs(
        report_id=report_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    report_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[Union[GetReportResponse404, Union["FinishedReport", "PendingReport"]]]:
    """Get Report

     Get a quality report by ID

    Args:
        report_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[GetReportResponse404, Union['FinishedReport', 'PendingReport']]
    """

    return sync_detailed(
        report_id=report_id,
        client=client,
    ).parsed


async def asyncio_detailed(
    report_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[Union[GetReportResponse404, Union["FinishedReport", "PendingReport"]]]:
    """Get Report

     Get a quality report by ID

    Args:
        report_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[GetReportResponse404, Union['FinishedReport', 'PendingReport']]]
    """

    kwargs = _get_kwargs(
        report_id=report_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    report_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[Union[GetReportResponse404, Union["FinishedReport", "PendingReport"]]]:
    """Get Report

     Get a quality report by ID

    Args:
        report_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[GetReportResponse404, Union['FinishedReport', 'PendingReport']]
    """

    return (
        await asyncio_detailed(
            report_id=report_id,
            client=client,
        )
    ).parsed



import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.algo_log_log import AlgoLogLog


T = TypeVar("T", bound="AlgoLog")


@_attrs_define
class AlgoLog:
    """
    Attributes:
        time (datetime.datetime):
        log (AlgoLogLog):
    """

    time: datetime.datetime
    log: "AlgoLogLog"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        time = self.time.isoformat()

        log = self.log.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "time": time,
                "log": log,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.algo_log_log import AlgoLogLog

        d = dict(src_dict)
        time = isoparse(d.pop("time"))

        log = AlgoLogLog.from_dict(d.pop("log"))

        algo_log = cls(
            time=time,
            log=log,
        )

        algo_log.additional_properties = d
        return algo_log

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
